﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace NSMBe4
{
    public partial class ASSettings : Form
    {
        private List<NSMBExtraData> dataList;
        private NSMBExtraData current;
        private bool acceptChanges = false;

        public ASSettings(LevelEditorControl ed)
        {
            dataList = ed.Level.ExtraData;

            InitializeComponent();
            updateList();
        }

        private void updateList()
        {
            acceptChanges = false;
            current = null;
            panel.Enabled = false;
            list.Items.Clear();
            for (int i = 0; i < dataList.Count; i++)
            {
                list.Items.Add("ID: " + i);
            }
            acceptChanges = true;
        }

        private void list_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (list.SelectedIndex == -1)
                return;

            panel.Enabled = true;
            current = dataList[list.SelectedIndex];
            ID.Value = current.classID;
            classIDName.SelectedIndex = current.classID;

            string sd = current.Data[0].ToString("X2");
            sd += " ";
            sd += current.Data[1].ToString("X2");
            sd += " ";
            sd += current.Data[2].ToString("X2");
            sd += " ";
            sd += current.Data[3].ToString("X2");
            spriteDataTextBox.Text = sd;

            acceptChanges = false;
            offsetX.Value = current.offsetX;
            offsetY.Value = current.offsetY;
            particleID.Value = current.particleID;
            particleIDName.SelectedIndex = current.particleID;
            if (current.particleID > particleIDName.Items.Count - 1)
                particleIDName.SelectedIndex = -1;
            else
                particleIDName.SelectedIndex = current.particleID;
            pOffsetX.Value = current.particleOffsetX;
            pOffsetY.Value = current.particleOffsetY;
            sfxID.Value = current.sfxID;
            if (current.sfxID > sfxIDName.Items.Count - 1)
                sfxIDName.SelectedIndex = -1;
            else
                sfxIDName.SelectedIndex = current.sfxID;
            acceptChanges = true;
        }

        private void add_Click(object sender, EventArgs e)
        {
            NSMBExtraData newData = new NSMBExtraData();
            newData.classID = 0;
            newData.Data = new byte[4];
            newData.offsetX = 0;
            newData.offsetY = 0;
            newData.particleID = 0;
            newData.sfxID = 0;

            dataList.Add(newData);
            updateList();
        }

        private void delete_Click(object sender, EventArgs e)
        {
            if (list.SelectedIndex == -1)
                return;

            dataList.RemoveAt(list.SelectedIndex);
            updateList();
        }

        private void spriteDataTextBox_TextChanged(object sender, EventArgs e)
        {
            if (!acceptChanges)
                return;

            acceptChanges = false;

            // validate
            if (System.Text.RegularExpressions.Regex.IsMatch(
                spriteDataTextBox.Text,
                "^[0-9a-f] *[0-9a-f] *[0-9a-f] *[0-9a-f] *[0-9a-f] *[0-9a-f] *[0-9a-f] *[0-9a-f] *$", System.Text.RegularExpressions.RegexOptions.IgnoreCase))
            {
                string parseit = spriteDataTextBox.Text.Replace(" ", "");
                byte[] data = new byte[4];
                for (int hexidx = 0; hexidx < 4; hexidx++)
                    data[hexidx] = byte.Parse(parseit.Substring(hexidx * 2, 2), System.Globalization.NumberStyles.AllowHexSpecifier);
                spriteDataTextBox.BackColor = SystemColors.Window;

                int currIndex = spriteDataTextBox.SelectionStart;
                parseit = parseit.ToUpper();
                parseit = parseit.Substring(0, 2) + " " + parseit.Substring(2, 2) + " " + parseit.Substring(4, 2) + " " + parseit.Substring(6, 2);
                spriteDataTextBox.Text = parseit;
                spriteDataTextBox.SelectionStart = currIndex;

                current.Data = data;
            }
            else
                spriteDataTextBox.BackColor = Color.Coral;

            acceptChanges = true;
        }

        private void offsetX_ValueChanged(object sender, EventArgs e)
        {
            if (!acceptChanges)
                return;

            current.offsetX = (sbyte)offsetX.Value;
        }

        private void offsetY_ValueChanged(object sender, EventArgs e)
        {
            if (!acceptChanges)
                return;

            current.offsetY = (sbyte)offsetY.Value;
        }

        private void particleID_ValueChanged(object sender, EventArgs e)
        {
            if (!acceptChanges)
                return;

            acceptChanges = false;
            if (current.particleID > particleIDName.Items.Count - 1)
                particleIDName.SelectedIndex = -1;
            else
                particleIDName.SelectedIndex = current.particleID;
            acceptChanges = true;

            current.particleID = (ushort)particleID.Value;
        }

        private void particleIDName_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!acceptChanges)
                return;

            acceptChanges = false;
            particleID.Value = particleIDName.SelectedIndex;
            acceptChanges = true;

            current.particleID = (ushort)particleIDName.SelectedIndex;
        }

        private void sfxID_ValueChanged(object sender, EventArgs e)
        {
            if (!acceptChanges)
                return;

            acceptChanges = false;
            if ((int)sfxID.Value > sfxIDName.Items.Count - 1)
                sfxIDName.SelectedIndex = -1;
            else
                sfxIDName.SelectedIndex = (int)sfxID.Value;
            acceptChanges = true;

            current.sfxID = (ushort)sfxID.Value;
        }

        private void sfxIDName_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!acceptChanges)
                return;

            acceptChanges = false;
            sfxID.Value = sfxIDName.SelectedIndex;
            acceptChanges = true;

            current.sfxID = (ushort)sfxIDName.SelectedIndex;
        }

        private void ID_ValueChanged(object sender, EventArgs e)
        {
            if (!acceptChanges)
                return;

            acceptChanges = false;
            classIDName.SelectedIndex = (int)ID.Value;
            acceptChanges = true;

            current.classID = (ushort)ID.Value;
        }

        private void classIDName_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!acceptChanges)
                return;

            acceptChanges = false;
            ID.Value = classIDName.SelectedIndex;
            acceptChanges = true;

            current.classID = (ushort)classIDName.SelectedIndex;
        }

        private void pOffsetX_ValueChanged(object sender, EventArgs e)
        {
            if (!acceptChanges)
                return;

            current.particleOffsetX = (sbyte)pOffsetX.Value;
        }

        private void pOffsetY_ValueChanged(object sender, EventArgs e)
        {
            if (!acceptChanges)
                return;

            current.particleOffsetY = (sbyte)pOffsetY.Value;
        }
    }
}
