#include <nds.h>
#include <nds/registers_alt.h>
#include "actorSpawner.h"
#include "nsmb.h"

bool eventIsActivated(u32 id)
{
	if (id == 0)
		return false;
	
	u64 *activatorBitmask = (u64*)(0x0208AF3C);
	return (*activatorBitmask >> (id-1)) & 1;
}

void ActorSpawner_doSpawn(ActorSpawner* as)
{
	Vec3 pos = as->position;
	pos.y -= 0x10000;		// Origin pos: Bottom-left of sprite

	pos.x += (s8)as->settings->offsetX * 0x1000;
	pos.y += (s8)as->settings->offsetY * 0x1000;
		
	if (as->sActor)
		createActor(as->settings->classID, as->settings->spriteData, &pos, 0, 0, 0);
		
	pos.x += (s8)as->settings->particleOffsetX * 0x1000;
	pos.y += (s8)as->settings->particleOffsetY * 0x1000;
		
	if (as->sParticles)
		summonParticle(as->settings->particleID, pos.x, pos.y, 0);
	if (as->sSFX)
		PlaySNDEffect(as->settings->sfxID, 0);
		
	as->actorSpawned = true;
}

void ActorSpawner_setup(ActorSpawner* as)
{	
	as->actorSpawned = false;
	
	u16 assID = (as->spriteData & 0XFF0000) >> 16;
	u32* pointerToBlock14 = (u32*)0x0208B19C;
	as->settings = (ActorSpawnerSettigs*)(*pointerToBlock14 + 16 + assID*16);
	
	as->sActor = (as->spriteData & 0x1000) == 0;
	as->sParticles = (as->spriteData & 0x2000) == 0;
	as->sSFX = (as->spriteData & 0x4000) == 0;
	as->sFirstTick = (as->spriteData & 0x8000) == 0;
	
	as->event = as->spriteData>>24;
	as->eventWasActive = false;
	as->spawnDelay = as->spriteData & 0xFFF;
	as->timer = 0;
}

void ActorSpawner_update_state0(ActorSpawner* as)
{
	if (as->event == 0 && as->spawnDelay == 0)
		return;
	
	if (as->spawnDelay == 0)
	{
		if (eventIsActivated(as->event) && as->actorSpawned)
			return;
		
		if (!eventIsActivated(as->event) && as->actorSpawned)
			as->actorSpawned = false;
		
		if (eventIsActivated(as->event) && !as->actorSpawned)
			ActorSpawner_doSpawn(as);
	}
	else
	{
		bool eventActive;
		if (as->event != 0)
			eventActive = eventIsActivated(as->event);
		else
			eventActive = true;
			
		if (as->eventWasActive && !eventActive)
			as->timer = 0;
			
		
		if (eventActive)
		{
			if (as->sFirstTick && !as->eventWasActive)
				ActorSpawner_doSpawn(as);
				
			if (as->timer > as->spawnDelay)
			{
				ActorSpawner_doSpawn(as);
				as->timer = 0;
			}	
			as->timer++;
		}
			
		as->eventWasActive = eventActive;	
	}
}